"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const C = require("@collectable/core");
const internals_1 = require("../internals");
const set_1 = require("./set");
/**
 * Creates a new `RedBlackTree` from an array of key/value pairs (tuples). If no ComparatorFn function is supplied, keys
 * are compared using logical less-than and greater-than operations, which will generally only be suitable for numeric
 * or string keys.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {[K, V][]} pairs An array of pairs (tuples), each being a two-element array of [key, value]
 * @param {ComparatorFn<K>} compare A comparison function, taking two keys, and returning a value less than 0 if the
 *                                  first key is smaller than the second, a value greater than 0 if the first key is
 *                                  greater than the second, or 0 if they're the same.
 * @returns {RedBlackTreeStructure<K, V>} A tree populated with an entry for each pair in the input array
 */
function fromPairs(compare, pairs, mutability) {
    const tree = C.modify(internals_1.createTree(compare, mutability));
    var pair;
    if (Array.isArray(pairs)) {
        for (var i = 0; i < pairs.length; i++) {
            pair = pairs[i];
            set_1.set(pair[0], pair[1], tree);
        }
    }
    else {
        const it = pairs[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            pair = current.value;
            set_1.set(pair[0], pair[1], tree);
        }
    }
    return C.commit(tree);
}
exports.fromPairs = fromPairs;
//# sourceMappingURL=fromPairs.js.map